@@ -52,7 +52,8 @@ def check_environment():
                     'https://newrelic.com/docs/python/python-agent-and-uwsgi.',
                     '.'.join(map(str, uwsgi.version_info[:3])))
 
-        if not uwsgi.opt.get('enable-threads'):
+        if (hasattr(uwsgi, 'opt') and hasattr(uwsgi.opt, 'get') and
+                not uwsgi.opt.get('enable-threads')):
             _logger.warning('The New Relic Python Agent requires that when '
                     'using uWSGI that the enable-threads option be given '
                     'to uwsgi when it is run. If the option is not supplied '
@@ -320,13 +321,40 @@ class Agent(object):
         settings = newrelic.core.config.global_settings()
 
         if timeout is None:
-            timeout = settings.startup_timeout
+            # In serverless mode, we should always wait for startup since it
+            # should take almost no time to create the session (no network
+            # activity).
+            if settings.serverless_mode.enabled:
+                timeout = 10.0
+            else:
+                timeout = settings.startup_timeout
 
         activate_session = False
 
         with self._lock:
             application = self._applications.get(app_name, None)
             if not application:
+
+                process_id = os.getpid()
+
+                if process_id != self._process_id:
+                    _logger.warning(
+                            'Attempt to activate application in a process '
+                            'different to where the agent harvest thread was '
+                            'started. No data will be reported for this '
+                            'process with pid of %d. Creation of the harvest '
+                            'thread this application occurred in process with '
+                            'pid %d. If no data at all is being reported for '
+                            'your application, see '
+                            'https://docs.newrelic.com/docs/agents/'
+                            'python-agent/troubleshooting/'
+                            'activate-application-warning-python '
+                            'for troubleshooting steps. If the issue '
+                            'persists, please send debug logs to New Relic '
+                            'support for assistance.',
+                            process_id,
+                            self._process_id)
+
                 if settings.debug.log_agent_initialization:
                     _logger.info('Creating application instance for %r '
                             'in process %d.', app_name, os.getpid())
@@ -364,13 +392,13 @@ class Agent(object):
 
                 application.validate_process()
 
-        # Activate the session if application was just created and wait
-        # for session activation if a timeout was specified. This may
-        # bail out early if is detected that a deadlock may occur for
-        # the period of the timeout.
+            # Activate the session if application was just created and wait
+            # for session activation if a timeout was specified. This may
+            # bail out early if is detected that a deadlock may occur for
+            # the period of the timeout.
 
-        if activate_session:
-            application.activate_session(timeout)
+            if activate_session:
+                application.activate_session(timeout)
 
     @property
     def applications(self):
@@ -514,6 +542,9 @@ class Agent(object):
 
         application.record_transaction(data, profile_samples)
 
+        if self._config.serverless_mode.enabled:
+            application.harvest()
+
     def normalize_name(self, app_name, name, rule_type='url'):
         application = self._applications.get(app_name, None)
         if application is None:
@@ -521,9 +552,9 @@ class Agent(object):
 
         return application.normalize_name(name, rule_type)
 
-    def compute_sampled(self, app_name, priority):
+    def compute_sampled(self, app_name):
         application = self._applications.get(app_name, None)
-        return application.compute_sampled(priority)
+        return application.compute_sampled()
 
     def _harvest_loop(self):
         _logger.debug('Entering harvest loop.')
@@ -639,6 +670,12 @@ class Agent(object):
         if not self._config.enabled:
             _logger.warning('The Python Agent is not enabled.')
             return
+        elif self._config.serverless_mode.enabled:
+            _logger.debug('Harvest thread is disabled due to serverless mode.')
+            return
+        elif self._config.debug.disable_harvest_until_shutdown:
+            _logger.debug('Harvest thread is disabled.')
+            return
 
         # Skip this if background thread already running.
 
@@ -649,6 +686,8 @@ class Agent(object):
 
         self._harvest_thread.start()
 
+        self._process_id = os.getpid()
+
     def _atexit_shutdown(self):
         """Triggers agent shutdown but flags first that this is being
         done because process is being shutdown.
@@ -668,7 +707,13 @@ class Agent(object):
         _logger.info('New Relic Python Agent Shutdown')
 
         self._harvest_shutdown.set()
-        self._harvest_thread.join(timeout)
+
+        if self._config.debug.disable_harvest_until_shutdown:
+            _logger.debug('Start Python Agent main thread on shutdown.')
+            self._harvest_thread.start()
+
+        if self._harvest_thread.is_alive():
+            self._harvest_thread.join(timeout)
 
 
 def agent_instance():
