@@ -28,6 +28,11 @@ except ImportError:
 DEFAULT_RESERVOIR_SIZE = 1200
 ERROR_EVENT_RESERVOIR_SIZE = 100
 
+# SPAN_EVENT_RESERVOIR_SIZE is both the default value for the span event
+# reservoir capacity, as well as its maximum possible value. server-side
+# config could adjust it lower, but not higher, than this value.
+SPAN_EVENT_RESERVOIR_SIZE = 1000
+
 # settings that should be completely ignored if set server side
 IGNORED_SERVER_SIDE_SETTINGS = ['utilization.logical_processors',
         'utilization.total_ram_mib', 'utilization.billing_hostname']
@@ -176,6 +181,34 @@ class HerokuSettings(Settings):
     pass
 
 
+class SpanEventSettings(Settings):
+    pass
+
+
+class SpanEventAttributesSettings(Settings):
+    pass
+
+
+class DistributedTracingSettings(Settings):
+    pass
+
+
+class ServerlessModeSettings(Settings):
+    pass
+
+
+class TransactionSegmentSettings(Settings):
+    pass
+
+
+class TransactionSegmentAttributesSettings(Settings):
+    pass
+
+
+class EventLoopVisibilitySettings(Settings):
+    pass
+
+
 _settings = Settings()
 _settings.attributes = AttributesSettings()
 _settings.thread_profiler = ThreadProfilerSettings()
@@ -187,6 +220,7 @@ _settings.browser_monitoring = BrowserMonitorSettings()
 _settings.browser_monitoring.attributes = BrowserMonitorAttributesSettings()
 _settings.transaction_name = TransactionNameSettings()
 _settings.transaction_metrics = TransactionMetricsSettings()
+_settings.event_loop_visibility = EventLoopVisibilitySettings()
 _settings.rum = RumSettings()
 _settings.slow_sql = SlowSqlSettings()
 _settings.agent_limits = AgentLimitsSettings()
@@ -208,6 +242,13 @@ _settings.datastore_tracer.instance_reporting = \
 _settings.datastore_tracer.database_name_reporting = \
         DatastoreTracerDatabaseNameReportingSettings()
 _settings.heroku = HerokuSettings()
+_settings.span_events = SpanEventSettings()
+_settings.span_events.attributes = SpanEventAttributesSettings()
+_settings.transaction_segments = TransactionSegmentSettings()
+_settings.transaction_segments.attributes = \
+        TransactionSegmentAttributesSettings()
+_settings.distributed_tracing = DistributedTracingSettings()
+_settings.serverless_mode = ServerlessModeSettings()
 
 _settings.log_file = os.environ.get('NEW_RELIC_LOG', None)
 _settings.audit_log_file = os.environ.get('NEW_RELIC_AUDIT_LOG', None)
@@ -221,6 +262,15 @@ def _environ_as_int(name, default=0):
         return default
 
 
+def _environ_as_float(name, default=0.0):
+    val = os.environ.get(name, default)
+
+    try:
+        return float(val)
+    except ValueError:
+        return default
+
+
 def _environ_as_bool(name, default=False):
     flag = os.environ.get(name, default)
     if default is None or default:
@@ -358,12 +408,17 @@ _settings.host = os.environ.get('NEW_RELIC_HOST',
         default_host(_settings.license_key))
 _settings.port = int(os.environ.get('NEW_RELIC_PORT', '0'))
 
+_settings.agent_run_id = None
+_settings.request_headers_map = {}
+
 _settings.proxy_scheme = os.environ.get('NEW_RELIC_PROXY_SCHEME', None)
 _settings.proxy_host = os.environ.get('NEW_RELIC_PROXY_HOST', None)
 _settings.proxy_port = int(os.environ.get('NEW_RELIC_PROXY_PORT', '0'))
 _settings.proxy_user = os.environ.get('NEW_RELIC_PROXY_USER', None)
 _settings.proxy_pass = os.environ.get('NEW_RELIC_PROXY_PASS', None)
 
+_settings.ca_bundle_path = os.environ.get('NEW_RELIC_CA_BUNDLE_PATH', None)
+
 _settings.app_name = os.environ.get('NEW_RELIC_APP_NAME', 'Python Application')
 _settings.linked_applications = []
 
@@ -383,10 +438,11 @@ _settings.attribute_filter = None
 _settings.collect_errors = True
 _settings.collect_error_events = True
 _settings.collect_traces = True
+_settings.collect_span_events = True
 _settings.collect_analytics_events = True
 _settings.collect_custom_events = True
 
-_settings.apdex_t = 0.5
+_settings.apdex_t = _environ_as_float('NEW_RELIC_APDEX_T', 0.5)
 _settings.web_transactions_apdex = {}
 
 _settings.capture_params = None
@@ -419,10 +475,17 @@ _settings.metric_name_rules = []
 _settings.transaction_name_rules = []
 _settings.transaction_segment_terms = []
 
-_settings.account_id = None
+_settings.account_id = os.environ.get('NEW_RELIC_ACCOUNT_ID')
 _settings.cross_process_id = None
+_settings.primary_application_id = \
+        os.environ.get('NEW_RELIC_PRIMARY_APPLICATION_ID', 'Unknown')
 _settings.trusted_account_ids = []
+_settings.trusted_account_key = os.environ.get('NEW_RELIC_TRUSTED_ACCOUNT_KEY')
 _settings.encoding_key = None
+_settings.sampling_target = 10
+_settings.sampling_target_period_in_seconds = 60
+
+_settings.max_payload_size_in_bytes = 1000000
 
 _settings.attributes.enabled = True
 _settings.attributes.exclude = []
@@ -441,6 +504,19 @@ _settings.transaction_events.attributes.include = []
 _settings.custom_insights_events.enabled = True
 _settings.custom_insights_events.max_samples_stored = DEFAULT_RESERVOIR_SIZE
 
+_settings.distributed_tracing.enabled = _environ_as_bool(
+        'NEW_RELIC_DISTRIBUTED_TRACING_ENABLED', default=False)
+_settings.span_events.enabled = _environ_as_bool(
+        'NEW_RELIC_SPAN_EVENTS_ENABLED', default=True)
+_settings.span_events.max_samples_stored = SPAN_EVENT_RESERVOIR_SIZE
+_settings.span_events.attributes.enabled = True
+_settings.span_events.attributes.exclude = []
+_settings.span_events.attributes.include = []
+
+_settings.transaction_segments.attributes.enabled = True
+_settings.transaction_segments.attributes.exclude = []
+_settings.transaction_segments.attributes.include = []
+
 _settings.transaction_tracer.enabled = True
 _settings.transaction_tracer.transaction_threshold = None
 _settings.transaction_tracer.record_sql = 'obfuscated'
@@ -492,7 +568,7 @@ _settings.agent_limits.max_sql_connections = 4
 _settings.agent_limits.sql_explain_plans = 30
 _settings.agent_limits.sql_explain_plans_per_harvest = 60
 _settings.agent_limits.slow_sql_data = 10
-_settings.agent_limits.merge_stats_maximum = 5
+_settings.agent_limits.merge_stats_maximum = None
 _settings.agent_limits.errors_per_transaction = 5
 _settings.agent_limits.errors_per_harvest = 20
 _settings.agent_limits.slow_transaction_dry_harvests = 5
@@ -504,8 +580,6 @@ _settings.agent_limits.synthetics_events = 200
 _settings.agent_limits.synthetics_transactions = 20
 _settings.agent_limits.data_compression_threshold = 64 * 1024
 _settings.agent_limits.data_compression_level = None
-_settings.agent_limits.max_outstanding_traces = 100
-_settings.agent_limits.sampling_target = 10
 
 _settings.console.listener_socket = None
 _settings.console.allow_interpreter_cmd = False
@@ -529,12 +603,15 @@ _settings.debug.record_transaction_failure = False
 _settings.debug.enable_coroutine_profiling = False
 _settings.debug.explain_plan_obfuscation = 'simple'
 _settings.debug.disable_certificate_validation = False
+_settings.debug.log_untrusted_distributed_trace_keys = False
+_settings.debug.disable_harvest_until_shutdown = False
 
 _settings.message_tracer.segment_parameters_enabled = True
 
 _settings.utilization.detect_aws = True
 _settings.utilization.detect_azure = True
 _settings.utilization.detect_docker = True
+_settings.utilization.detect_kubernetes = True
 _settings.utilization.detect_gcp = True
 _settings.utilization.detect_pcf = True
 
@@ -556,6 +633,14 @@ _settings.heroku.use_dyno_names = _environ_as_bool(
 _settings.heroku.dyno_name_prefixes_to_shorten = list(_environ_as_set(
         'NEW_RELIC_HEROKU_DYNO_NAME_PREFIXES_TO_SHORTEN', 'scheduler run'))
 
+_settings.serverless_mode.enabled = _environ_as_bool(
+        'NEW_RELIC_SERVERLESS_MODE_ENABLED',
+        default=False)
+_settings.aws_arn = None
+
+_settings.event_loop_visibility.enabled = True
+_settings.event_loop_visibility.blocking_threshold = 0.1
+
 
 def global_settings():
     """This returns the default global settings. Generally only used
@@ -760,19 +845,18 @@ def apply_server_side_settings(server_side_config={}, settings=_settings):
         if value == 'apdex_f':
             agent_config['transaction_tracer.transaction_threshold'] = None
 
-    # Overlay with global server side configuration settings.
-
-    for (name, value) in server_side_config.items():
-        apply_config_setting(settings_snapshot, name, value)
-
     # Overlay with agent server side configuration settings.
-    # Assuming for now that agent service side configuration
-    # can always take precedence over the global server side
-    # configuration settings.
 
     for (name, value) in agent_config.items():
         apply_config_setting(settings_snapshot, name, value)
 
+    # Overlay with global server side configuration settings.
+    # global server side configuration always takes precedence over the global
+    # server side configuration settings.
+
+    for (name, value) in server_side_config.items():
+        apply_config_setting(settings_snapshot, name, value)
+
     # This will be removed at some future point
     # Special case for account_id which will be sent instead of
     # cross_process_id in the future
