@@ -1,10 +1,10 @@
-import newrelic.api.web_transaction
+import newrelic.api.wsgi_application
 import newrelic.api.in_function
 
 def instrument_meinheld_server(module):
 
     def wrap_wsgi_application_entry_point(application, *args, **kwargs):
-        application = newrelic.api.web_transaction.WSGIApplicationWrapper(
+        application = newrelic.api.wsgi_application.WSGIApplicationWrapper(
                 application)
         args = [application] + list(args)
         return (args, kwargs)
