@@ -32,7 +32,7 @@
 
 from newrelic.api.function_trace import FunctionTrace, FunctionTraceWrapper
 from newrelic.api.transaction import current_transaction
-from newrelic.api.web_transaction import wrap_wsgi_application
+from newrelic.api.wsgi_application import wrap_wsgi_application
 from newrelic.common.object_names import callable_name
 from newrelic.common.object_wrapper import (FunctionWrapper, wrap_out_function,
         wrap_function_wrapper)
@@ -87,7 +87,7 @@ def view_handler_wrapper(wrapped, instance, args, kwargs):
 
     transaction.set_transaction_name(name)
 
-    with FunctionTrace(transaction, name):
+    with FunctionTrace(name):
         try:
             return wrapped(*args, **kwargs)
 
@@ -135,7 +135,7 @@ def default_view_mapper_wrapper(wrapped, instance, args, kwargs):
 
         name = callable_name(view)
 
-        with FunctionTrace(transaction, name=name) as tracer:
+        with FunctionTrace(name=name) as tracer:
             try:
                 return wrapper(context, request)
             finally:
